/***** drawGenealogy.c ********************************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mgp.de
 * File created on Fri Sep 23 09:37:15 2005.
 *****************************************************************/
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include "interface.h"
#include "eprintf.h"
#include "stringUtil.h"
#include "drawGenealogy.h"

int main(int argc, char *argv[]){
  Args *args;          /* arguments */
  char *version;       /* program version */
  int i, j, k, count, tia, tmrca;
  const gsl_rng_type * T;
  gsl_rng *ranf;
  Indiv ***pop;
  FILE *fp;
  int pick1, pick2, empty, full;

  version = "1.1";

  args = getArgs(argc, argv);
  if(args->h == 1){
    printUsage(version);
    return 0; 
  }else if(args->e == 1){
    printUsage(version);
    return -1;
  }
  setprogname2("drawGenealogy");
  /* setup random number generator */
  gsl_rng_env_setup(); 
  T = gsl_rng_default;

  ranf = gsl_rng_alloc (T);
  if(args->s == 0)
    gsl_rng_set(ranf, time(NULL));
  else
    gsl_rng_set(ranf, args->s);

  /* generate population making sure that there is at least one male & one female*/
  pop = (Indiv ***)emalloc(args->g*sizeof(Indiv **));
  for(i=args->g-1;i>=0;i--){
    pop[i] = (Indiv **)emalloc(args->p*sizeof(Indiv *));
    count = 0;
    for(j=0;j<args->p;j++){
      pop[i][j] = getIndiv(ranf,args,i,j);
      if(pop[i][j]->isMale)
	count++;
    }
    if(count == args->p)
      pop[i][0]->isMale = 0;
    else if(count == 0)
      pop[i][0]->isMale = 1;
  }
  for(i=0;i<args->p;i++)
    pop[args->g-1][i]->descendants[i] = 1;
  /* generate genealogy */
  for(i=args->g-1;i>0;i--){
    for(j=0;j<args->p;j++){
      /* let descendants pick ancestors */
      pick1 = gsl_rng_uniform(ranf)*args->p;
      pick2 = gsl_rng_uniform(ranf)*args->p;
/*       while((pick2=gsl_rng_uniform(ranf)*args->p) == pick1) */
/* 	; */
      while(pop[i-1][pick2]->isMale == pop[i-1][pick1]->isMale)
	pick2 = gsl_rng_uniform(ranf)*args->p;
      pop[i][j]->ancestor1 = pop[i-1][pick1];
      pop[i][j]->ancestor2 = pop[i-1][pick2];
      if(gsl_rng_uniform(ranf) < 0.5)
	pop[i][j]->gene1->ancestor = pop[i-1][pick1]->gene1;
      else
	pop[i][j]->gene1->ancestor = pop[i-1][pick1]->gene2;
      if(gsl_rng_uniform(ranf) < 0.5)
	pop[i][j]->gene2->ancestor = pop[i-1][pick2]->gene1;
      else
	pop[i][j]->gene2->ancestor = pop[i-1][pick2]->gene2;
      /* inherit descendants */
      for(k=0;k<args->p;k++){
	if(pop[i][j]->descendants[k]){
	  pop[i-1][pick1]->descendants[k] = 1;
	  pop[i-1][pick2]->descendants[k] = 1;
	}	
      }
    }
  }
  /* check for universal ancestors & universal non-ancestors*/
  for(i=0;i<args->g;i++){
    for(j=0;j<args->p;j++){
      count = 0;
      for(k=0;k<args->p;k++){
	if(pop[i][j]->descendants[k]){
	  count++;
	}
      }
      if(count == args->p){
	pop[i][j]->isUa = 1;
      }else if(count == 0){
	pop[i][j]->isNonUa = 1;
      }
    }
  }
  /* compute tmrca */
  k = 0;
  for(i=args->g-1;i>0;i--){
    for(j=0;j<args->p;j++)
      if(pop[i][j]->isUa){
	pop[i][j]->isMrca = 1;
	k = 1;
      }
    if(k)
      break;
  }
  tmrca = args->g - 1 - i;
  /* compute tia */
  for(i=args->g-1;i>0;i--){
    full = empty = 0;
    for(j=0;j<args->p;j++)
      if(pop[i][j]->isUa)
	full++;
      else if(pop[i][j]->isNonUa)
	empty++;
    if(full > 0 && full+empty == args->p)
      break;
  }
  tia = args->g - 1 - i;
  if(args->c){
    printf("TMRCA = %d\n",tmrca);
    printf("TIA = %d\n",tia);
  }else{
    draw(args,pop);
    if(args->t == NULL)
      printf("To view the graphic in file %s, include it in a LaTeX file.\n",args->o);
    else{
      fp = efopen(args->t,"w");
      printLatexHeader(fp);
      fprintf(fp,"\\begin{center}\\resizebox{\\textwidth}{!}{\\input{");
      i = 0;
      j = strlen(args->o);
      while(args->o[i] != '.' && i < j)
	fprintf(fp,"%c",args->o[i++]);
      fprintf(fp,"}}\\end{center}\n\n\\end{document}");
      printf("To view the graphic, run latex %s\n",args->t);
      fclose(fp);
    }
  }
  return 0;
}


