/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, bernhard.haubold@fh-weihenstephan.de
 * File created on Sun Jun 20 13:12:10 2004.
 *
 * This file is part of drawGenealogy.
 *
 *   drawWrightFisher is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   drawGenealogy is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with drawGenealogy; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <limits.h>
#include "interface.h"
#include "eprintf.h"

Args *getArgs(int argc, char *argv[]){
  Args *args;
  char c;
  int arg; 

  args = (Args *)emalloc(sizeof(Args));

  args->p = -1;         /* population size */
  args->n = -1;         /* sample size */
  args->o = "genealogy.tex";       /* output file */
  args->t = NULL;
  args->g = 10;

  args->d = args->D = INT_MAX;
  args->s = 0;
  args->C = 0;
  args->c = 0;
  args->i = 0;
  args->h = 0;
  args->e = 1;
  arg = 1;
  while(arg < argc){
    c = argv[arg][1];
    switch(c){
    case 'n':
      args->n = (int)atoi(argv[++arg]);   /* sample size */
      break;
    case 'p':
      args->p = (int)atoi(argv[++arg]);   /* population size */
      args->e = 0;
      break;
    case 'g':                            /* generations */
      args->g = (int)atoi(argv[++arg]);  
      break;
    case 'd':                            /* draw number of generations for individuals */
      args->d = (int)atoi(argv[++arg]);  
      break;
    case 'D':                            /* draw number of generations for genes */
      args->D = (int)atoi(argv[++arg]);  
      break;
    case 'i':                            /* individual */
      args->i = (int)atoi(argv[++arg]);  
      break;
    case 'o':                            /* output file */
      args->o = argv[++arg];  
      break;
    case 't':                            /* test file */
      args->t = argv[++arg];  
      break;
    case 'C':                           /* Color code individuals */
      args->C = 1;
      break;
    case 'c':                           /* computations only */
      args->c = 1;
      break;
    case 's':                            /* seed for random number generator */
      args->s = (int)atoi(argv[++arg]);
      if(args->s < 0)
	args->s = -args->s;
      break;
    case 'h':                            /* print help */
      args->h = 1;
      break;
    default:
      printf("# unknown argument: %c\n",c);
      args->e = 1;
      return args;
    }
    arg++;
  }

  args->d = args->g - args->d;
  args->D = args->g - args->D;
  return args;
}

void printUsage(char *version){
  printf("drawGenealogy version %s, copyright (c) 2006 Bernhard Haubold\n", version);
  printf("purpose: draw genealogy of diploid individual in Wright-Fisher population\n");
  printf("arguments:  \n");
  printf("\t-p NUM population size=NUM\n");
  printf("\t[-g NUM run for NUM generations; default: NUM=10]\n");
  printf("\t[-i NUM draw genealogy for individual NUM; default: all]\n");
  printf("\t[-d NUM draw <= NUM generations for individuals; default: all]\n");
  printf("\t[-D NUM draw <= NUM generations for genes; default: all]\n");
  printf("\t[-s NUM use NUM as seed for random number generator; default: system clock]\n");
  printf("\t[-o FILE write output to FILE; default: FILE genealogy.tex]\n");
  printf("\t[-t FILE write test file to FILE; default: no test file]\n");			       
  printf("\t[-C color code individuals; default: b/w line drawing]\n");
  printf("\t[-c computations only ; default: draw]\n");
  printf("\t[-h print this help message]\n");											
}
 
